print <<'HEADER';
# $Id: Base.pm,v 1.1.1.1 2001/05/18 16:36:37 matt Exp $
# version 0.10 - Kip Hampton <khampton@totalcinema.com>
# version 0.13 - Robin Berjon <robin@knowscape.com>
# version 0.15 - Kip Hampton <khampton@totalcinema.com>

package XML::Filter::Base;

use strict;
use vars qw($VERSION $DEBUG $LEXICAL_HANDLER $DECL_HANDLER);

$VERSION = '0.15';

HEADER

#BEGIN {
    $DEBUG = 0;
    $LEXICAL_HANDLER = 'http://xml.org/sax/handlers/LexicalHandler';
    $DECL_HANDLER    = 'http://xml.org/sax/handlers/DeclHandler';
    my %EVENT_SPEC = (
                        start_document          => [qw(ContentHandler DocumentHandler Handler)],
                        end_document            => [qw(ContentHandler DocumentHandler Handler)],
                        start_element           => [qw(ContentHandler DocumentHandler Handler)],
                        start_document          => [qw(ContentHandler DocumentHandler Handler)],
                        end_document            => [qw(ContentHandler DocumentHandler Handler)],
                        start_element           => [qw(ContentHandler DocumentHandler Handler)],
                        end_element             => [qw(ContentHandler DocumentHandler Handler)],
                        characters              => [qw(ContentHandler DocumentHandler Handler)],
                        processing_instruction  => [qw(ContentHandler DocumentHandler Handler)],
                        ignorable_whitespace    => [qw(ContentHandler DocumentHandler Handler)],
                        set_document_locator    => [qw(ContentHandler DocumentHandler Handler)],
                        start_prefix_mapping    => [qw(ContentHandler Handler)],
                        end_prefix_mapping      => [qw(ContentHandler Handler)],
                        skipped_entity          => [qw(ContentHandler Handler)],
                        start_cdata             => ['DocumentHandler', $LEXICAL_HANDLER],
                        end_cdata               => ['DocumentHandler', $LEXICAL_HANDLER],
                        comment                 => ['DocumentHandler', $LEXICAL_HANDLER],
                        entity_reference        => [qw(DocumentHandler Handler)],
                        notation_decl           => [qw(DTDHandler)],
                        unparsed_entity_decl    => [qw(DTDHandler)],
                        element_decl            => [qw(DTDHandler)],
                        attlist_decl            => [qw(DTDHandler)],
                        doctype_decl            => [qw(DTDHandler)],
                        xml_decl                => [qw(DTDHandler)],
                        entity_decl             => [$DECL_HANDLER, 'DTDHandler'],
                        attribute_decl          => [$DECL_HANDLER, 'Handler'],
                        internal_entity_decl    => [$DECL_HANDLER, 'Handler'],
                        external_entity_decl    => [$DECL_HANDLER, 'Handler'],
                        resolve_entity          => [qw(EntityResolver)],
                        start_dtd               => [$LEXICAL_HANDLER],
                        end_dtd                 => [$LEXICAL_HANDLER],
                        start_entity            => [$LEXICAL_HANDLER],
                        end_entity              => [$LEXICAL_HANDLER],
                        warning                 => [qw(ErrorHandler)],
                        error                   => [qw(ErrorHandler)],
                        fatal_error             => [qw(ErrorHandler)],
                     );
    my $code;
    for my $ev (keys %EVENT_SPEC) {
        $code .= <<"        EOTOPCODE";
sub $ev {
    my \$self = shift;
    my \$ret;
        EOTOPCODE

        for my $h (@{$EVENT_SPEC{$ev}}) {
            $code .= <<"            EOINCODE";
    eval { \$ret = \$self->{'$h'}->$ev(\@_); } if \$self->{'$h'};
    return \$ret if \$self->{'$h'} and not \$\@;
    die \$\@ if \$\@ and \$\@ !~ m/Can't locate object method "$ev"/;
            EOINCODE
        }
        $code .= "    undef \$\@;\n";
        $code .= "    return;\n}\n\n";
    }

print $code;

print <<'BODY';
sub new {
    my $class = shift;
    my %options = @_;

# we could set the SAX version
#    if ($options{ContentHandler})     { $options{SAXVersion} = 2; }
#    elsif ($options{DocumentHandler}) { $options{SAXVersion} = 1; }
    $options{Handler}           ||= XML::Filter::Base::NoHandler->new;
    $options{$LEXICAL_HANDLER}  ||= $options{Handler};
    $options{DTDHandler}        ||= $options{Handler};
    $options{EntityResolver}    ||= $options{Handler};
    $options{ErrorHandler}      ||= $options{Handler};

# these are not defaulted (for now at least)
#    $options{DocumentHandler}   ||= $options{Handler};
#    $options{ContentHandler}    ||= $options{Handler};
#    $options{$DECL_HANDLER}     ||= $options{Handler};

    return bless \%options, $class;
}

package XML::Filter::Base::NoHandler;

# we need a fake handler that doesn't implement anything, this
# simplifies the code a lot (though given the recent changes,
# it may be better to do without)
sub new {
    return bless {};
}


1;

=pod

=head1 NAME

XML::Filter::Base - Base class for XML::Filter modules

=head1 SYNOPSIS

  package MyFilter;
  use XML::Filter::Base;
  @ISA = ('XML::Filter::Base');

=head1 DESCRIPTION

This module has a very simple task - to be a base class for PerlSAX
filters. It's default behaviour is to pass the input directly to the
output unchanged. It can be useful to use this module as a base class
so you don't have to, for example, implement the characters() callback.

=head1 METHODS

It would be rather useless to describe all the methods that this
module implements here. They are all the methods supported in SAX1 and
SAX2. In case your memory is a little short, here is a list. The
apparent duplicates are there so that both versions of SAX can be
supported.

=over 4

=item * start_document

=item * end_document

=item * start_element

=item * start_document

=item * end_document

=item * start_element

=item * end_element

=item * characters

=item * processing_instruction

=item * ignorable_whitespace

=item * set_document_locator

=item * start_prefix_mapping

=item * end_prefix_mapping

=item * skipped_entity

=item * start_cdata

=item * end_cdata

=item * comment

=item * entity_reference

=item * notation_decl

=item * unparsed_entity_decl

=item * element_decl

=item * attlist_decl

=item * doctype_decl

=item * xml_decl

=item * entity_decl

=item * attribute_decl

=item * internal_entity_decl

=item * external_entity_decl

=item * resolve_entity

=item * start_dtd

=item * end_dtd

=item * start_entity

=item * end_entity

=item * warning

=item * error

=item * fatal_error

=back

=head1 TODO

  - more tests
  - conform to the "SAX Filters" and "Java and DOM compatibility"
  sections of the SAX2 document.

=head1 AUTHOR

Matt Sergeant, matt@sergeant.org

=head1 SEE ALSO

L<XML::Parser::PerlSAX>.

=cut

BODY
