#!/usr/bin/perl -w

use strict;
use XML::Parser::PerlSAX;
use XML::Handler::YAWriter;
use IO::File;

my $file = $ARGV[0] || die "Please pass a file name to process\n";

my $writer = XML::Handler::YAWriter->new(Output => IO::File->new( ">-" ));
my $filter = FilterPorcus->new(Handler => $writer);
my $parser = XML::Parser::PerlSAX->new(Handler => $filter);

my %parser_args = (Source => {SystemId => $file});
$parser->parse(%parser_args);

# end main

# silly text transformer
package FilterPorcus;
use strict;
use base qw(XML::Filter::Base);

sub new {
    my $class = shift;
    my %options = @_;
    return bless \%options, $class;
}

sub characters {
    my ($self, $chars) = @_;
    my $out = $self->porcus($chars->{Data});
    $self->{Handler}->characters({Data => $out});
}

sub start_element {
    my ($self, $element) = @_;
    my %attrs = %{$element->{Attributes}};

    while ( my ($name, $value) = (each (%attrs))) {
        my $orig_name = $name;
        $name = $self->porcus($name);
        $value = $self->porcus($value);
        $attrs{$name} = $value;
        delete $attrs{$orig_name};
    }

    $element->{Attributes} = \%attrs;
    my $elname = $self->porcus($element->{Name});
    $element->{Name} = $elname;
    $self->{Handler}->start_element($element);
}

sub end_element {
    my ($self, $element) = @_;
    my $elname = $self->porcus($element->{Name});
    $element->{Name} = $elname;
    $self->{Handler}->end_element($element);
}


sub porcus {
    my ($self, $chars) = @_;
    $chars =~ tr/A-Z/a-z/;
    $chars =~ s/\b([aeiou])/w$1/g;
    my $cons = q{[bcfghjklmnpqrstvwxz]};
    $chars =~ s/\b(qu|$cons($cons$cons?)?|[a-z])([a-z]*)/$3$1ay/g;
    return $chars;
}
1;
