
#!/usr/bin/perl -w

use strict;
use XML::Parser::PerlSAX;
use XML::Filter::SAX2toSAX1;
use XML::Filter::SAX1toSAX2;
use XML::Handler::YAWriter;
use IO::File;

my $file = $ARGV[0] || die "Please pass a file name to process\n";

my $writer = XML::Handler::YAWriter->new(Output => IO::File->new( ">-" ));
$writer->{Pretty}->{NoProlog} = 1;
my $sax1_filter = XML::Filter::SAX2toSAX1->new(Handler => $writer);
my $handler = FilterXInclude->new(Handler => $sax1_filter);
my $sax2_filter = XML::Filter::SAX1toSAX2->new(Handler => $handler);
my $parser = XML::Parser::PerlSAX->new(Handler => $sax2_filter);

my %parser_args = (Source => {SystemId => $file});
$parser->parse(%parser_args);

# end main

# minimal XInclude Implementation
package FilterXInclude;
use strict;
use base qw(XML::Filter::Base);
use XML::Parser::PerlSAX;
use XML::Filter::SAX2toSAX1;
use XML::Filter::SAX1toSAX2;

#use vars qw(@ISA);
#@ISA = qw(XML::Filter::Base);

sub new {
    my $class = shift;
    my %options = @_;
    $options{BaseURI} ||= './';
    return bless \%options, $class;
}


sub start_element {
    my ($self, $element) = @_;
    my %attrs = %{$element->{Attributes}};

    if (defined $attrs{'{http://www.w3.org/XML/1998/namespace}base'}) {
       $self->{BaseURI} =  $attrs{'{http://www.w3.org/XML/1998/namespace}base'}->{Value};
       $self->{BaseURI} =~ s|^file://||;
    }

    if ($element->{NamespaceURI} eq 'http://www.w3.org/2001/XInclude'
        and $element->{LocalName} eq 'include') {
        $self->include_proc($attrs{'{}href'}->{Value});
    }
    else {
        $self->{Handler}->start_element($element);
    }
}

sub end_element {
    my ($self, $element) = @_;	
    unless ($element->{NamespaceURI} eq 'http://www.w3.org/2001/XInclude'
        and $element->{LocalName} eq 'include') {
        $self->{Handler}->end_element($element);
    }
}

sub include_proc {
    my ($self, $file) = @_;
    $file = $self->{BaseURI} . $file;
    my $sax2_filter = XML::Filter::SAX1toSAX2->new(Handler => $self);
    my $parser = XML::Parser::PerlSAX->new({Handler => $sax2_filter, Source => {SystemId => $file}});
    $parser->parse;
}

1;
